/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.component.smartclient.spi;

import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;

import org.objectweb.easybeans.component.smartclient.client.AskingClassLoader;

/**
 * Initial Context factory used on the client side.<br>
 * It will ask the server for every class/resource not found in local.
 * @author Florent Benoit
 *
 */
public class SmartContextFactory implements InitialContextFactory {

    /**
     * Use the JDK logger (to avoid any dependency).
     */
    private static Logger logger = Logger.getLogger(SmartContextFactory.class.getName());

    /**
     * Default PROVIDER_URL.
     */
    private static final String DEFAULT_URL = "smart://localhost:2503";

    /**
     * Carol factory.
     */
    private static final String CAROL_FACTORY = "org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory";

    /**
     * EasyBeans delegating factory. (by default it will be ourself)
     */
    public static final String EASYBEANS_DELEGATING_FACTORY = "easybeans.smart.delegate.factory";

    /**
     * EasyBeans factory.
     */
    public static final String EASYBEANS_FACTORY = "easybeans.rpc.rmi.factory";

    /**
     * Instance of classloader that is used to load class.<br>
     * This is this classloader which will download class.
     */
    private static AskingClassLoader classLoader = null;

    /**
     * Provider URL to use for connecting.
     */
    private static String providerURL = null;

    /**
     * Default constructor.<br>
     * Sets the Portable RemoteObject Wrapper class too.
     */
    public SmartContextFactory() {
        // sets our class (or the delegating factory)
        System.setProperty(EASYBEANS_FACTORY, System.getProperty(EASYBEANS_DELEGATING_FACTORY, this.getClass().getName()));
        System.setProperty("javax.rmi.CORBA.PortableRemoteObjectClass", ProDelegate.class.getName());
    }

    /**
      * Creates an Initial Context for beginning name resolution.
      * Special requirements of this context are supplied
      * @param environment the given environment.
      * @return the context.
      * @throws NamingException if no context can be built.
    */
    @SuppressWarnings("unchecked")
    public Context getInitialContext(final Hashtable environment)
            throws NamingException {

        if (classLoader == null) {
            try {
                // Gets the provider url
                String currentProviderURL = (String) environment.get(Context.PROVIDER_URL);
                if (currentProviderURL == null) {
                    logger.log(Level.WARNING, "No PROVIDER_URL setting found, use the default URL '" + DEFAULT_URL + "'.");
                    currentProviderURL = DEFAULT_URL;
                }

                logger.log(Level.INFO, "Initializing Smart Factory with remote URL '" + currentProviderURL + "'.");

                // extract host
                String host = getHostOfUrl(currentProviderURL);

                // extract port
                int portNumber = getPortOfUrl(currentProviderURL);

                // build the classloader to use.
                classLoader = new AskingClassLoader(host, portNumber);


                providerURL = classLoader.getProviderURL();
                logger.log(Level.INFO, "Got remote PROVIDER_URL '" + providerURL + "'.");

                // Set the classloader for the ProDelegate class
                ProDelegate.setClassLoader(classLoader);

            } catch (Exception e) {
                NamingException ne = new NamingException("Cannot get a remote ClassLoader");
                ne.initCause(e);
                throw ne;
            }
        }

        ClassLoader old = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(classLoader);
        try {
            // set the right factory
            environment.put(Context.INITIAL_CONTEXT_FACTORY, CAROL_FACTORY);
            environment.put(Context.PROVIDER_URL, providerURL);

            // return wrapped context
            return new SmartContext(new InitialContext(environment), classLoader);
        } finally {
            Thread.currentThread().setContextClassLoader(old);
        }

    }


    /**
     * Parses the given url, and returns the port number. 0 is given in error
     * case)
     * @param url given url on which extract port number
     * @return port number of the url
     * @throws NamingException if URL is invalid
     */
    public int getPortOfUrl(final String url) throws NamingException {
        int portNumber = 0;
        try {
            StringTokenizer st = new StringTokenizer(url, ":");
            st.nextToken();
            st.nextToken();
            if (st.hasMoreTokens()) {
                StringTokenizer lastst = new StringTokenizer(st.nextToken(), "/");
                String pts = lastst.nextToken().trim();
                int i = pts.indexOf(',');
                if (i > 0) {
                    pts = pts.substring(0, i);
                }
                portNumber = new Integer(pts).intValue();
            }
            return portNumber;
        } catch (Exception e) {
            // don't rethrow original exception. only URL name is important
            throw new NamingException("Invalid URL '" + url + "'. It should be on the format <protocol>://<hostname>:<port>");
        }
    }

    /**
     * Parses the given url, and returns the hostname.
     * @param url given url on which extract hostname
     * @return hostname of the url
     * @throws NamingException if URL is invalid
     */
    private String getHostOfUrl(final String url) throws NamingException {
        String host = null;
        // this would be simpler with a regexp :)
        try {
            // url is of the form protocol://<hostname>:<port>
            String[] tmpSplitStr = url.split(":");

            // array should be of length = 3
            // get 2nd element (should be //<hostname>)
            String tmpHost = tmpSplitStr[1];

            // remove //
            String[] tmpSplitHost = tmpHost.split("/");

            // Get last element of the array to get hostname
            host = tmpSplitHost[tmpSplitHost.length - 1];
        } catch (Exception e) {
            // don't rethrow original exception. only URL name is important
            throw new NamingException("Invalid URL '" + url + "'. It should be on the format <protocol>://<hostname>:<port>");
        }
        return host;
    }
}
