/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.log;

import java.io.Serializable;
import java.text.MessageFormat;
import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.easybeans.i18n.I18n;

/**
 * Logger with I18n and infinite arguments.<br>
 * @author Florent Benoit
 */
public class JDKLogger implements JLog, Serializable {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = -3786299204538237422L;

    /**
     * Name of this logger.
     */
    private String name = null;

    /**
     * Logger object (transient).
     */
    private transient Logger logger = null;

    /**
     * I18n object.
     */
    private I18n i18n = null;

    /**
     * Handler already set.
     */
    private static boolean handlerSet = false;

    /**
     * Property to disable the set of our own handler.
     */
    public static final String SET_HANDLER = "easybeans.log.handler";

    /**
     * Creates a new logger.
     * @param clazz the class that requires a logger.
     * @param i18n the internationalization object to use, if any.
     */
    public JDKLogger(final Class clazz, final I18n i18n) {
        this.name = clazz.getName();
        this.i18n = i18n;
        logger = getLogger();

    }

    /**
     * Log the given message with the given throwable (if any).
     * @param level the level of the message (to log)
     * @param msg the content of the message
     * @param t the throwable (if any)
     */
    private void log(final Level level, final String msg, final Throwable t) {
        // Get stack trace
        Throwable throwable = new Throwable();
        StackTraceElement[] stackTraceElements = throwable.getStackTrace();

        // Caller = 3rd element in the stack
        String className = "stack-not-found";
        String methodName = "stack-not-found";
        if (stackTraceElements != null && stackTraceElements.length > 2) {
            StackTraceElement caller = stackTraceElements[2];
            className = caller.getClassName();
            methodName = caller.getMethodName();
        }

        // Get logger
        Logger logger = getLogger();

        // log
        if (t == null) {
            logger.logp(level, className, methodName, msg);
        } else {
            logger.logp(level, className, methodName, msg, t);
        }

    }

    /**
     * Replace the console handler by our handler.
     * @param logger the given logger to analyze.
     */
    private void replaceConsoleHandlerInParentHandlers(final Logger logger) {
        if (logger == null) {
            return;
        }
        Logger parent =  logger.getParent();
        if (parent == null) {
            return;
        }
        Handler[] handlers = parent.getHandlers();
        if (handlers != null) {
            for (Handler handler : handlers) {
                if (handler instanceof ConsoleHandler) {
                    parent.removeHandler(handler);
                    // add our customize handler
                    Handler easyBeansHandler = new JDKConsoleHandler();
                    easyBeansHandler.setFormatter(new JDKFormatter());
                    parent.addHandler(easyBeansHandler);
                }
           }
        }
    }

    /**
     * @return JDK logger
     */
    public Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger(name);
        }


        // Remove parent handler if not already done
        if (!handlerSet) {
            String prop = System.getProperty(SET_HANDLER, "true");
            // if true, set our own handler
            if (Boolean.parseBoolean(prop)) {
                replaceConsoleHandlerInParentHandlers(logger);

            }
            // even if it was false, set value to true to avoid to check each time.
            handlerSet = true;
        }
        return logger;
    }

    /**
     * <p>
     * Is debug logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than debug.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isDebugEnabled() {
        return (getLogger().isLoggable(Level.FINE));
    }

    /**
     * <p>
     * Is error logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than error.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isErrorEnabled() {
        return (getLogger().isLoggable(Level.SEVERE));
    }

    /**
     * <p>
     * Is fatal logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than fatal.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isFatalEnabled() {
        return (getLogger().isLoggable(Level.SEVERE));
    }

    /**
     * <p>
     * Is info logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than info.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isInfoEnabled() {
        return (getLogger().isLoggable(Level.INFO));
    }

    /**
     * <p>
     * Is trace logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than trace.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isTraceEnabled() {
        return (getLogger().isLoggable(Level.FINEST));
    }

    /**
     * <p>
     * Is warn logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than warn.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isWarnEnabled() {
        return (getLogger().isLoggable(Level.WARNING));
    }

    /**
     * Gets the i18n object associated to this logger.
     * @return i18n object.
     */
    public I18n getI18n() {
        return i18n;
    }

    /**
     * <p>
     * Log a message with debug log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void debug(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!isDebugEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        log(Level.FINE, msg, t);

    }

    /**
     * <p>
     * Log a message with error log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void error(final Object message, final Object... args) {
        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        log(Level.SEVERE, msg, t);
    }

    /**
     * <p>
     * Log a message with fatal log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void fatal(final Object message, final Object... args) {
        // same as error
        error(message, args);
    }

    /**
     * <p>
     * Log a message with info log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void info(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!isInfoEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        log(Level.INFO, msg, t);
    }

    /**
     * <p>
     * Log a message with trace log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void trace(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!isTraceEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        log(Level.FINEST, msg, t);

    }

    /**
     * <p>
     * Log a message with warn log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void warn(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!isWarnEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        log(Level.WARNING, msg, t);

    }

    /**
     * Gets a string message build with the given args.<br> *
     * @param message If the message is an i18n entry, this entry will be used
     *        to format the message.<br>
     *        Else, the arguments will be used as string formatter
     * @param args given args use by the formatter
     * @return a string representation of the given message.
     */
    private String getMessage(final Object message, final Object... args) {

        // I18n present ?
        if (i18n != null) {
            // try to format message
            return i18n.getMessage(String.valueOf(message), args);
        }

        // else, try to format message
        return MessageFormat.format(String.valueOf(message), args);
    }

    /**
     * Try to find a Throwable object in the array of object args. The throwable
     * should be the last parameter.
     * @param args array of objects on which we need to find the Throwable
     * @return a throwable object if last index if a Throwable or null
     */
    private Throwable getThrowable(final Object... args) {
        int size = args.length;
        // Detect the size
        if (size > 0) {
            Object lastArg = args[size - 1];
            if (lastArg instanceof Throwable) {
                return (Throwable) lastArg;
            }
        }

        // not found
        return null;
    }

}
