/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EZBDirectoryArchiveImpl.java 1026 2006-08-04 14:54:10Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.archive;

import static org.objectweb.easybeans.util.url.URLUtils.fileToURL2;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.easybeans.api.EZBArchive;
import org.objectweb.easybeans.api.EZBArchiveException;
import org.objectweb.easybeans.util.url.URLUtilsException;
/**
 * Creates wrapper around directory.
 * @author Florent Benoit
 */
public class EZBDirectoryArchiveImpl implements EZBArchive {

    /**
     * Internal resource used as archive.
     */
    private File directory = null;

    /**
     * Creates new instance of an EZBArchive for a directory.
     * @param directory the given directory.
     */
    protected EZBDirectoryArchiveImpl(final File directory) {
        this.directory = directory;
    }

    /**
     * @return a description of this archive. This name could be used in logger info.
     */
    public String getName() {
        return directory.getPath();
    }

    /**
     * Encode resource name to be used on Unix/Windows systems.
     * @param resourceName the name to encode.
     * @return the encoded name.
     */
    private String encode(final String resourceName) {
        String[] tokens = resourceName.split("/");
        StringBuilder sb = new StringBuilder();
        for (String token : tokens) {
            if (sb.length() > 0) {
                sb.append(File.separator);
            }
            sb.append(token);
        }
        return sb.toString();
    }


    /**
     * Close the underlying Resource.
     * @return Returns <code>true</code> if the close was
     *         succesful, <code>false</code> otherwise.
     */
    public boolean close() {
        // Nothing to close
        return true;
    }

    /**
     * @param resourceName The resource name to be looked up.
     * @return Returns the resource URL if the resource
     *         has been found. null otherwise.
     * @throws EZBArchiveException if method fails.
     */
    public URL getResource(final String resourceName) throws EZBArchiveException {
        URL resourceURL = null;
        // lookup the directory on filesystem
        File f = new File(directory, encode(resourceName));
        try {
            if (f.exists()) {
                resourceURL = fileToURL2(f);
            }
        } catch (URLUtilsException e) {
            throw new EZBArchiveException("Invalid url", e);
        }
        return resourceURL;
    }

    /**
     * @return Returns an Iterator of Resource's URL.
     * @throws EZBArchiveException if method fails.
     */
    public Iterator<URL> getResources() throws EZBArchiveException {
        List<URL> listResources = new ArrayList<URL>();
        // Get all files and subdirectories
        addFiles(directory, listResources);
        return listResources.iterator();
    }

    /**
     * Methods that loop on directories to find the children (files).
     * @param file the given directory/file.
     * @param listResources the list on which to add new files.
     */
    private void addFiles(final File file, final List<URL> listResources) {
        if (!file.exists()) {
            return;
        }

        if (file.isDirectory()) {
            // directory
            File[] files = file.listFiles();
            for (File f : files) {
                // loop
                addFiles(f, listResources);
            }
        } else {
            // single file
            try {
                listResources.add(fileToURL2(file));
            } catch (URLUtilsException e) {
                throw new IllegalStateException("Invalid url", e);
            }
        }
    }


    /**
     * @param resourceName The resource name to be looked up.
     * @return Returns an Iterator of matching resources.
     * @throws EZBArchiveException if method fails.
     */
    public Iterator<URL> getResources(final String resourceName) throws EZBArchiveException{
        List<URL> listResources = new ArrayList<URL>();
        File f = new File(directory, encode(resourceName));
        if (f.exists()) {
            try {
                listResources.add(fileToURL2(f));
            } catch (URLUtilsException e) {
                throw new EZBArchiveException("Invalid url", e);
            }
        }

        return listResources.iterator();

    }

    /**
     * @return Returns the resource URL.
     * @throws EZBArchiveException if method fails.
     */
    public URL getURL() throws EZBArchiveException {
        try {
            return fileToURL2(directory);
        } catch (URLUtilsException e) {
            throw new IllegalStateException("Invalid url", e);
        }
    }

    /**
     * Is that the given object is equals to our instance.
     * @param o the object to compare.
     * @return true if equals, else false.
     */
    @Override
    public boolean equals(final Object o) {
        if (!(o instanceof EZBDirectoryArchiveImpl)) {
            return false;
        }
        EZBDirectoryArchiveImpl other = (EZBDirectoryArchiveImpl) o;
        return this.directory.equals(other.directory);
    }

    /**
     * Gets hashcode for this object.
     * @return hash code.
     */
    @Override
    public int hashCode() {
        return directory.hashCode();
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        return getName();
    }
}
