/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: StatefulSessionFactory.java 940 2006-07-26 09:05:27Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.session.stateful;

import java.util.HashMap;
import java.util.Map;

import javax.ejb.NoSuchEJBException;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.FactoryException;
import org.objectweb.easybeans.api.bean.EasyBeansSFSB;
import org.objectweb.easybeans.api.pool.PoolException;
import org.objectweb.easybeans.container.session.SessionFactory;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.pool.JPool;
import org.objectweb.easybeans.pool.PoolEntryStatistics;
import org.objectweb.easybeans.pool.PoolFactory;

/**
 * This class manages the stateless session bean and its creation/lifecycle.
 * @author Florent Benoit
 */
public class StatefulSessionFactory extends SessionFactory<EasyBeansSFSB> implements
        PoolFactory<EasyBeansSFSB, Long> {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(StatefulSessionFactory.class);

    /**
     * Id generator.
     */
    private Long idCount = Long.valueOf(0L);

    /**
     * Id of created objects.
     */
    private Long idCreated = Long.valueOf(0L);


    /**
     * Builds a new factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    public StatefulSessionFactory(final String className, final EZBContainer container) throws FactoryException {
        super(className, container);
        setPool(new JPool<EasyBeansSFSB, Long>(this));
    }

    /**
     * Checks if the given object with the given clue is matching.
     * @param bean given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    public boolean isMatching(final EasyBeansSFSB bean, final Long clue) {
        logger.debug("Called with bean = {0} and beanId = {1}", bean, clue);
        Long beanId = idMap.get(bean);
        boolean val = (beanId.equals(clue));
        logger.debug("Found id = {0} and will return = {1}", beanId, Boolean.valueOf(val));

        return val;
    }

    /**
     * Validate an instance by giving some statistics.
     * @param object the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    public boolean validate(final EasyBeansSFSB object, final PoolEntryStatistics stats) {
        return true;
    }

    /**
     * TODO: to be removed.
     * Map between bean and its id
     */
    private Map<EasyBeansSFSB, Long> idMap = new HashMap<EasyBeansSFSB, Long>();


    /**
     * Gets a new ID or a null value.
     * @param beanId given id.
     * @return new id
     */
    @Override
    protected Long getId(final Long beanId) {
        Long newId = beanId;
        // no Id, compute a new one
        if (newId == null) {
            idCount = Long.valueOf(idCount.longValue() + 1);
            newId = idCount;
        }
        return newId;
    }


    /**
     * Creates an instance with the given hint.
     * @param clue a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    @Override
    public EasyBeansSFSB create(final Long clue) throws PoolException {
        // verify that clue is valid.
        // If we try to create an object with an already created clue, it means that object has been removed.
        // Exception needs to be thrown.
        if (idCreated.longValue() >= clue.longValue()) {
            throw new NoSuchEJBException("Object has been removed");
        }

        // else, create it
        EasyBeansSFSB bean = super.create(clue);

        // object was created, update counter
        idCreated = idCount;
        return bean;
    }

    /**
     * Gets a bean for the given id.
     * @param beanId id of the expected bean.
     * @return a Stateless bean.
     * @throws IllegalArgumentException if bean is not found.
     */
    @Override
    protected EasyBeansSFSB getBean(final Long beanId) throws IllegalArgumentException {
        EasyBeansSFSB bean = null;
        try {
            bean = getPool().get(beanId);
        } catch (PoolException e) {
            throw new IllegalArgumentException("Cannot get element in the pool", e);
        }
        //TODO : remove this temp hack. id should be in the bean itself.
        logger.debug("Add in the map bean {0} with beanId = {1}", bean, beanId);
        idMap.put(bean, beanId);
        return bean;
    }


}
